var app = {

	notification_api: 'http://117.53.163.75/stormsafe-mobile-app/api/index.php',
	analytics_tracking_id: 'UA-34307249-7', 
	gcm_sender_id: '261920140524', 

	device_id: null, 
	notification_id: null, 
	platform: null, // 1: Android, 2: iOS
	position: null, 
	region_code: null, 

	active_class: 'active', 
	ajax_timeout: 8000, 

	$pages: null, 
	$page_tabs: null, 

	$window: null, 
	$document: null, 
	$html: null, 
	$region_search: null, 
	$landing_page_bg: null, 

	search_results_template: null, 
	region_template: null, 
	news_template: null, 

	save_progress_request: null, 
	get_app_data_request: null, 
	get_news_request: null, 
	region_search_request: null, 

	start_at_contacts_subtab: false, // if set to true go straight to the Contacts sub-tab of region 

	stringReplaceAll: function(str, find_str, replace_str) {
	 	return (str || '').replace(new RegExp(find_str, 'g'), replace_str);
	}, 

	onRegister: function(registration_id) {
		if(registration_id == 'OK') return;

		if(app.device_id) {
			if(registration_id != app.notification_id) {
				// the notification_id has changed, update the app and notify the server

				app.notification_id = registration_id;
				window.localStorage.setItem('notification_id', app.notification_id);

				$.ajax({
					url: app.notification_api, 
					type: 'post', 
					timeout: app.ajax_timeout, 
					data: {
						'action': 'update_notification_id', 
						'device_id': app.device_id, 
						'notification_id': app.notification_id, 
					}
				});
			}
		}
		else {
			// we don't have a device id, so this must be the first time the user has used 
			// the app, or their window.localStorage was cleared for some reason

			app.notification_id = registration_id;
			window.localStorage.setItem('notification_id', app.notification_id);

			$.ajax({
				url: app.notification_api, 
				type: 'post', 
				timeout: app.ajax_timeout, 
				data: {
					'action': 'register_device', 
					'notification_id': app.notification_id, 
					'platform': app.platform
				}, 
				dataType: 'json', 
				success: function(response) {
					app.device_id = response || null;
					window.localStorage.setItem('device_id', app.device_id);
					if(app.device_id) analytics.setUserId(app.device_id);
				}
			});
		}
	}, 
	onRegisterError: function(error) {
		//app.showAlert('Registration Error', error || '');
	}, 

	onNotification: function() {
		app.$page_tabs.filter('[href="#news"]').addClass('alert');
	}, 
	onNotificationGCM: function(e) {
		if(e.event == 'message') app.onNotification();
		else if(e.event == 'registered' && e.regid.length > 0) app.onRegister(e.regid);
	}, 
	onNotificationAPN: function(event) {
		if(event.alert) app.onNotification();
	}, 

	showAlert: function(title, message, button, callback) {
		navigator.notification.alert(
			message || '', 
			callback || null, 
			title || 'Notification', 
			button || 'OK'
		);
	}, 

	openSystemBrowser: function(url) {
		return window.open(url, '_system', 'location=yes');
	}, 

	onGetPosition: function(position) {
		app.position = position;
        alert("onGetPosition");

		app.$region_search.prop('placeholder', 'Finding closest suburb...');

		if(app.$region_search.val() == '') {
			$.ajax({
				url: app.notification_api, 
				type: 'post', 
				timeout: app.ajax_timeout, 
				data: {
					'action': 'get_suburb_from_latlng', 
					'device_id': app.device_id, 
					'lat': position.coords.latitude, 
					'lng': position.coords.longitude
				}, 
				success: function(response) {
					if(app.$region_search.val() == '') {
						app.$region_search.val(response);
						app.onSearchRegion();
					}
				}, 
				error: function(xhr, text_status, error_thrown) {
                    if(text_status != 'abort') {
						if(app.$region_search.val() == '') {
							app.showAlert(
								'Geolocation Error', 
								'Could not determine your closest suburb. Please try searching manually.'
							);
						}
					}
				}, 
				complete: function() {
					app.$region_search.prop('placeholder', app.$region_search.data('placeholder'));
				}
			});
		}
	}, 
	onGetPositionError: function(error) {
		app.showAlert('Geolocation Error', error.message || '');
		app.$region_search.prop('placeholder', app.$region_search.data('placeholder'));
	}, 

	onResume: function() {
		setTimeout(function() { // wrapped in a setTmeout to prevent app hanging in iOS
			if(app.hasNetworkConnection) {
				app.getNews();
				app.getAppData();
			}
		}, 0);
	}, 

	getNews: function() {
		if(!app.hasNetworkConnection) return; //FIXME: notification?

		if(app.get_news_request || false) {
			app.get_news_request.abort();
		}

		app.get_news_request = $.ajax({
			url: app.notification_api, 
			type: 'post', 
			timeout: app.ajax_timeout, 
			data: {
				'action': 'get_news', 
				'device_id': app.device_id, 
				'region_code': app.region_code
			}, 
			dataType: 'json', 
			success: function(response) {
				var $news_articles = $('#news-articles');
				var $articles = $news_articles.find('article');

				if(response.news.length) {
					var is_updated = false;

					if($articles.length == response.news.length) {
						var old_article_ids = [];

						$articles.each(function() {
							old_article_ids.push(parseInt(this.id.replace('article-', '')));
						});

						for(var index in response.news) {
							if(old_article_ids.indexOf(response.news[index].id) < 0) {
								is_updated = true;
							}
						}
					}
					else is_updated = true;

					if(is_updated) {
						$news_articles.html(app.news_template.render(response));

						var $news_btn = app.$page_tabs.filter('[href="#news"]');
						if(!$news_btn.hasClass('active')) $news_btn.addClass('alert');

						window.localStorage.setItem('news', JSON.stringify(response));
					}
				}
				else $news_articles.html('<p class="warning">No notifications to display. Please check back later.</p>');
			}
		});
	}, 

	onSearchRegion: function() {
		var query = '' + (app.$region_search.val() || '');

		if(query.length > 3) {
			var $results = $('#landing-page ul').html('').show();
			$results.html('<li class="searching">Searching...</li>');

			if(app.region_search_request || false) {
				app.region_search_request.abort();
			}

			app.region_search_request = $.ajax({
				url: app.notification_api, 
				type: 'post', 
				timeout: app.ajax_timeout, 
				data: {
					'action': 'region_search', 
					'query': query
				}, 
				dataType: 'json', 
				success: function(response) {
					var response_html = response.map(function(result) {
						return app.search_results_template.render(result);
					});

					$results.html(response_html.join('\n'));
				}, 
				error: function(xhr, text_status, error_thrown) {
                    if(text_status != 'abort') {
						$results.html('<li class="error">No results found.</li>');
					}
				}
			});
		}

		return false;
	}, 

	trackView: function() {
		try {
			var $current_view = app.$pages.filter('.' + app.active_class);

			var tracking_str = $.trim('' + $current_view.find('.header').text());
			if(tracking_str == 'StormSafe') {
				var $region_header = $current_view.find('.region-page.' + app.active_class + ' .header');
				if($region_header.length) tracking_str = $.trim($region_header.text());
				else tracking_str = 'Region Search';
			}

			var $current_sub_view = $current_view.find('.subtabs .' + app.active_class);
			if($current_sub_view.length) tracking_str += ': ' + $.trim($current_sub_view.text());

			analytics.trackView(tracking_str);
		}
		catch(e) { }
	}, 

	activateSubTab: function(subtab_id) {
		var $subtab_panel = $(subtab_id);

		if(!$subtab_panel.hasClass(app.active_class)) {
			var $subtab_tabs = $subtab_panel.closest('.page, .region-page').find('.subtabs');

			$subtab_panel.addClass(app.active_class).siblings().removeClass(app.active_class);
			$subtab_tabs.find('.active').removeClass(app.active_class);
			$subtab_tabs.find('a[href="' + subtab_id + '"]').addClass(app.active_class);

			window.scrollTo(0, 0);
			app.trackView();
		}
	}, 

	onShowPage: function() {
		var $this = app.$pages.filter('#' + this.id);

		app.$page_tabs.removeClass(app.active_class);
		app.$page_tabs.filter('[href="#' + this.id + '"]').addClass(app.active_class);

		app.$pages.removeClass(app.active_class);
		$this.addClass(app.active_class);

		switch(this.id) {
			case 'local-info':
				if(app.$region_search.closest('.subtab-panel').hasClass(app.active_class)) {
					app.$region_search.prop('placeholder', 'Detecting your location...');
					navigator.geolocation.getCurrentPosition(app.onGetPosition, app.onGetPositionError);

					setTimeout(function() {
						if(app.$region_search.prop('placeholder') == 'Detecting your location...') {
							// GPS detection was cancelled or timed out
							app.$region_search.prop('placeholder', app.$region_search.data('placeholder'));
						}
					}, 10000);
				}
				else if(app.start_at_contacts_subtab) {
					app.activateSubTab($this.find('a.contacts-subtab-btn').attr('href'));
					app.start_at_contacts_subtab = false;
				}
			break;

			case 'news':
				app.getNews();
				app.$page_tabs.filter('[href="#news"]').removeClass('alert');
			break;
		}

		window.scrollTo(0, 0);
		app.trackView();
	}, 

	onClickPageTab: function() {
		if((this.class || '').indexOf(app.active_class) < 0) {
			app.$pages.filter(this.hash).trigger('showPage');
		}
		return false;
	}, 

	onDeviceReady: function() {
		var pushNotification = window.plugins.pushNotification;

 		app.platform = device.platform.toLowerCase() == 'android' ? 1 : 2;

		if(app.platform == 1) { // Android
			// Android <= 2.3 can't display SVGs
			if(parseInt(device.version) <= 2) app.$html.addClass('old-android');

			pushNotification.register(app.onRegister, app.onRegisterError, {
				'senderID': app.gcm_sender_id, 
    			'ecb': 'app.onNotificationGCM'
			});
		}
		else { // iOS
			// iOS has problems with displaying position:fixed elements when the virtual keyboard is visible
			app.$document.on('focus', 'input', function(e) { app.$html.addClass('fixfixed'); })
				.on('blur', 'input', function(e) { app.$html.removeClass('fixfixed'); });

			pushNotification.register(app.onRegister, app.onRegisterError, {
				'badge': 'true', 
				'sound': 'true', 
				'alert': 'true', 
    			'ecb': 'app.onNotificationAPN'
			});
		}

		analytics.startTrackerWithId(app.analytics_tracking_id);
		if(app.device_id) analytics.setUserId(app.device_id);

		app.$page_tabs.first().click();

		// REQUEST FROM NRMA: make sure the splashscreen displays for at least 2 seconds
		setTimeout(function() { navigator.splashscreen.hide(); }, 2500);
	}, 

	onGetRegionData: function(response) {
		app.region_code = response.code || null;

		if(app.start_at_contacts_subtab) {
			response.contacts = true;
			app.start_at_contacts_subtab = false;
		}

		var region_html = app.region_template.render(response);
		var $landing_page = $('#landing-page');

		$landing_page.removeClass(app.active_class);
		$(region_html).addClass(app.active_class).insertAfter($landing_page);
	}, 

	getRegionData: function(region_name) {
		if(app.hasNetworkConnection) {
			$.ajax({
				url: app.notification_api, 
				type: 'post', 
				timeout: app.ajax_timeout, 
				data: {
					'action': 'get_region', 
					'region': region_name, 
					'device_id': app.device_id
				}, 
				dataType: 'json', 
				success: app.onGetRegionData, 
				error: function(xhr, text_status, error_thrown) {
                    if(text_status != 'abort') {
						app.showAlert('Update Error', 'Could not retrieve the latest data for your region. Please try again in a few minutes.');
						app.resetLandingPage();
					}
				}
			});
		}
		else app.showAlert('Connection Error', 'Please connect to the internet and try again.');
	}, 

	hasNetworkConnection: function() {
		return navigator.network.connection.type !== Connection.NONE;
	}, 

	savePrepareProgress: function(e) {
		var reminder_clicked = e && e.target && e.target.id && e.target.id == 'reminder';

		var the_date = $('#checklist input[type="date"]').val();
		if(the_date != '') {
			// check if system supports datepicker, otherwise rely on the server to validate

			var not_a_date_value = ':-(';
			var date_input = document.createElement('input');

			date_input.setAttribute('type', 'date');
			date_input.setAttribute('value', not_a_date_value);

			if(date_input.value !== not_a_date_value) {
				the_date = new Date(the_date.replace(/(\d{2})-(\d{2})-(\d{4})/, "$2/$1/$3"));

				if(the_date < (new Date())) {
					app.showAlert('Invalid Date', 'Please select an insurance renewal date in the future.');
					return false;
				}
			}
		}

		if(app.save_progress_request || false) {
			app.save_progress_request.abort();
		}

		if(reminder_clicked) {
			try { analytics.trackEvent('Button Clicked', 'Update Insurance Reminder'); }
			catch(e) {}
		}

		var progress = {};
		$('#checklist input').each(function() {
			if(this.type == 'checkbox') progress[this.name] = this.checked ? 1 : 0;
			else progress[this.name] = $(this).val();
		});

		app.save_progress_request = $.ajax({
			url: app.notification_api, 
			type: 'post', 
			timeout: app.ajax_timeout, 
			data: {
				'action': 'save_prepare_progress', 
				'progress': progress, 
				'device_id': app.device_id, 
				'platform': app.platform
			}, 
			success: function(response, status, xhr) {
				// if the device_id was just registered update the app device_id
				if(xhr.status == 201) {
					app.device_id = response || null;
					window.localStorage.setItem('device_id', app.device_id);
					if(app.device_id) analytics.setUserId(app.device_id);
				}

				if(reminder_clicked) {
					app.showAlert('Reminder Set', 'Your insurance reminder has been updated');
				}
			}, 
			error: function(xhr, text_status, error_thrown) {
                if(text_status != 'abort') {
	 				if(reminder_clicked) {
						app.showAlert('Connection Error', 'Could not update your insurance reminder. Please try again in a few minutes. ' + xhr.responseText);
					}
				}
			}
		});
	}, 

	onGetAppData: function() {
		var response = JSON.parse(window.localStorage.getItem('app_data') || '');

		for(var key in response) {
			if(key == 'prepare' || key == 'cached') continue;

			var $content_area = $('#' + response[key].url);

			if($content_area.length) {
				$content_area.html(response[key].content);
				$content_area.find('img').wrap('<div class="img-container"></div>');
			}
		}

		if(response.prepare || false) {
			var $steps_template = $('#steps-template');
			var steps_html = Hogan.compile($steps_template.html()).render(response.prepare);
			$steps_template.siblings().remove();
			$steps_template.before(steps_html);

			// add the insurance date reminder form
			var renewal_date = response.prepare['insurance-due'] || '';
			var today = (new Date()).toISOString().slice(0, 10);
			$(
				'<div class="content-block">'
					+ '<label>Please select the renewal date for your insurance.</label>'
					+ '<input type="date" name="insurance_due" min="' + today + '" value="' + renewal_date + '" />'
					+ '<a class="button-secondary" id="reminder">' + (renewal_date.length ? 'Update' : 'Set') + ' Reminder</a>'
				+ '</div>'
			)
			.appendTo('#prepare-step-5 .article-content > div')
			.find('a').on('click', app.savePrepareProgress);

			// add the home emergency plan button
			$('#prepare-step-6 .article-content > div')
				.append('<a href="http://www.seshomeemergencyplan.com.au/" class="button-secondary">Prepare Home Emergency Plan</a>');

			// add the emergency kit button
			$('<a href="#prepare-kit" class="button-secondary subtab-link">Prepare an Emergency Kit</a>')
				.appendTo('#prepare-step-7 .article-content > div');

			// add the radio stations button
			$('<a class="button-secondary">Find Local Stations</a>')
				.on('click', function() { $('#btn-local-contacts').click(); })
				.appendTo('#prepare-step-8 .article-content > div');
		}
	}, 

	getAppData: function() {
		if(app.get_app_data_request || false) {
			app.get_app_data_request.abort();
		}

		if(app.hasNetworkConnection) {
			app.get_app_data_request = $.ajax({
				url: app.notification_api, 
				type: 'post', 
				timeout: app.ajax_timeout, 
				data: {
					'action': 'get_app_data', 
					'device_id': app.device_id
				}, 
				dataType: 'json', 
				success: function(response) {
					window.localStorage.setItem('app_data', JSON.stringify(response));
				}, 
				error: function(xhr, text_status, error_thrown) {
                    if(text_status != 'abort') {
						var app_data = window.localStorage.getItem('app_data') || '';
						if(app_data == '') app.showAlert('Update Error', 'Could not retrieve the latest app data. Please try again in a few minutes.');
                    }
				}, 
				complete: app.onGetAppData
			});
		}
		else {
			app.showAlert('Connection Error', 'Could not load the latest data. Please connect to the internet and try again.');
			app.onGetAppData();
		}
	}, 

	resetLandingPage: function() {
		var $local_info = app.$pages.filter('#local-info');

		app.$region_search.val('');
		app.$region_search.closest('form').next().hide();

		$local_info.find('.region-page').removeClass(app.active_class);
		$local_info.find('#landing-page').addClass(app.active_class);
		$local_info.trigger('showPage');

		return false;
	}, 

	initialize: function() {
		app.device_id = window.localStorage.getItem('device_id') || null;
		app.notification_id = window.localStorage.getItem('notification_id') || null;

		app.$window = $(window);
		app.$document = $(document);
		app.$html = $('html');
		app.$pages = $('.page').on('showPage', app.onShowPage);
		app.$page_tabs = $('#page-tabs a').on('click', app.onClickPageTab);
		app.$region_search = $('#region-search');
		app.$landing_page_bg = $('#landing-page-bg');

		app.$region_search.data('placeholder', app.$region_search.prop('placeholder'));
		app.$region_search.closest('form').on('submit', app.onSearchRegion);

		document.addEventListener('deviceready', app.onDeviceReady, false);
		document.addEventListener('resume', app.onResume, false);

		FastClick.attach(document.body);

		app.$pages.on('click', '.header a[href="#landing-page"]', app.resetLandingPage);

		app.$pages.filter('#local-info').on('click', 'a[data-region]', function() {
			app.getRegionData($(this).data('region'));
			return false;
		});

		app.$pages.on('click', '.subtabs a, .subtab-link', function() {
			app.activateSubTab($(this).attr('href'));
			return false;
		});

		app.$pages.on('click', '.article-header input', function() {
			try {
				var status = this.checked ? 'Checked' : 'Unchecked';
				analytics.trackEvent('Prepare Checklist', status, this.id);
			}
			catch(e) {}

			app.savePrepareProgress();
		});

		app.$pages.on('click', '.article-header', function(e) {
			if(!e.target.id) var $parent = $(this).parent().toggleClass('open');
		});

		$('#btn-local-contacts').click(function() {
			app.start_at_contacts_subtab = true;
			app.$pages.filter('#local-info').trigger('showPage');
			return false;
		});

		app.$pages.on('click', 'a[href^="http"]', function() {
			try { analytics.trackEvent('External Link', this.href); }
			catch(e) {}

			return !app.openSystemBrowser(this.href);
		});

		app.$pages.on('click', 'a[href^="tel:"]', function() {
			try { analytics.trackEvent('Button Clicked', 'Call Phone Number', this.href); }
			catch(e) {}

			return !app.openSystemBrowser(this.href);
		});

		app.search_results_template = Hogan.compile($('#search-results-template').html());
		app.region_template = Hogan.compile($('#region-template').html());
		app.news_template = Hogan.compile($('#news-template').html());

		app.getAppData();

		var news_json = JSON.parse(window.localStorage.getItem('news') || '');
		$('#news-articles').html(app.news_template.render(news_json));
		app.getNews();
	}

};
